USE tempdb;
SET NOCOUNT ON;
GO

/* Date/time shorthand QUIZ */

-- demo 1 :

DECLARE @d date = '20170407'--GETDATE();

SELECT 
  'DATEPART( D )', DATEPART( D , @d )    
  --   (a)  7      (b)  6    (c)  5
UNION ALL
SELECT                                                
  'DATEPART( W )', DATEPART( W , @d )   
  --   (a)  8      (b)  6      (c)  7
UNION ALL
SELECT                               
  'DATEPART( M )', DATEPART( M , @d )    
  --   (a)  4      (b)  This minute
UNION ALL
SELECT                                                     'DATEPART( Y )', DATEPART( Y , @d );
  --   (a)  17     (b)  97     (c)  2017














-- Don't use shorthand! If you mean MONTH, type MONTH
















-- demo 2 :

-- this works ok:
DECLARE @dt datetime = CURRENT_TIMESTAMP;
SELECT @dt + 1;
GO

-- this breaks:
DECLARE @d datetime2 = CURRENT_TIMESTAMP;
SELECT @d + 1;
GO

-- always use explicit DATEADD













/* Regional formats */

-- SET LANGUAGE FRENCH;

SELECT CONVERT(datetime, '2012.09.08')
UNION ALL
SELECT CONVERT(datetime, '09/08/2012')
UNION ALL
SELECT CONVERT(datetime, '2012-09-08 12:34:56.789');

-- safe:

SELECT CONVERT(datetime, '20120908')
UNION ALL
SELECT CONVERT(datetime, '2012-09-08T12:34:56.789')
UNION ALL
SELECT CONVERT(datetime, '20120908 12:34:56.789');

-- SET LANGUAGE US_ENGLISH;






/* BETWEEN / "end" of period */

CREATE TABLE dbo.SalesOrders
(
  OrderDate datetime2
);
GO

INSERT dbo.SalesOrders(OrderDate) VALUES
  ('20170201 00:00'),
  ('20170201 01:00'),
  ('20170219 00:00'),
  ('20170228 04:00'),
  ('20170228 13:27:32.534'),
  ('20170228 23:59:59.9999999'),
  ('20170301 00:00');
GO

SELECT OrderDate 
  FROM dbo.SalesOrders
  ORDER BY OrderDate;
GO

-- parameters are datetime, using old millisecond tricks:
DECLARE 
  @start datetime = '20170201',
  @end   datetime = DATEADD(MILLISECOND, -3, '20170301');

SELECT OrderDate, @start, @end 
  FROM dbo.SalesOrders
  WHERE OrderDate BETWEEN @start AND @end;
GO

-- parameters change to smalldatetime:
DECLARE 
 @start smalldatetime = '20170201',
 @end   smalldatetime = DATEADD(MILLISECOND, -3, '20170301');

SELECT OrderDate, @start, @end 
  FROM dbo.SalesOrders
  WHERE OrderDate BETWEEN @start AND @end;
GO

-- parameters change to date:
DECLARE 
  @start date = '20170201',
  @end   date = DATEADD(MILLISECOND, -3, '20170301');

SELECT OrderDate, @start, @end 
  FROM dbo.SalesOrders
  WHERE OrderDate BETWEEN @start AND @end;
GO

-- how about EOMONTH()?
DECLARE 
  @start date = '20170201';

SELECT OrderDate, @start, EOMONTH(@start) 
  FROM dbo.SalesOrders
  WHERE OrderDate BETWEEN @start AND EOMONTH(@start);
GO

-- Stop using BETWEEN. Open-ended range is safer,
-- and far easier to calculate the "end" of a period
DECLARE 
  @start date = '20170201';

SELECT OrderDate, @start, DATEADD(MONTH, 1, @start)
  FROM dbo.SalesOrders
  WHERE OrderDate >= @start 
    AND OrderDate < DATEADD(MONTH, 1, @start);
GO

DROP TABLE dbo.SalesOrders;
GO




/* FORMAT() */

DBCC FREEPROCCACHE;
GO
DECLARE @x char(8);
SELECT @x = FORMAT(modify_date, 'yyyyMMdd') 
  FROM sys.all_objects;
GO
DECLARE @x char(8);
SELECT @x = CONVERT(CHAR(8), modify_date, 112) 
  FROM sys.all_objects;
GO

-- compare performance:
SELECT CASE WHEN t.[text] LIKE N'%FORMAT(%' 
   THEN 'Format' ELSE 'Convert' END, 
   qs.total_elapsed_time 
FROM sys.dm_exec_query_stats AS qs
CROSS APPLY sys.dm_exec_sql_text(qs.plan_handle) AS t
WHERE t.[text] LIKE N'%@'+N'x = %';









/* Non-sargable expressions */

SELECT DISTINCT TOP (50000) 
    id = o.object_id, c.column_id, o.modify_date
  INTO dbo.objects
  FROM sys.all_objects AS o
  CROSS JOIN sys.all_columns AS c;
GO
CREATE UNIQUE CLUSTERED INDEX x 
  ON dbo.objects(id, column_id);
GO
CREATE INDEX y ON dbo.objects(modify_date);
GO

SELECT id FROM dbo.objects 
  WHERE YEAR(modify_date) = 2017 
    AND MONTH(modify_date) = 1;

SELECT id FROM dbo.objects 
  WHERE CONVERT(char(7), modify_date, 120) 
      = CONVERT(char(7), GETDATE(), 120);

SELECT id FROM dbo.objects 
  WHERE modify_date >= '20170101' 
    AND modify_date < '20170201';

GO
DROP TABLE dbo.objects;