USE tempdb;
SET NOCOUNT ON;
GO
/* ORDER BY myths */

CREATE TABLE dbo.what
(
  ID int PRIMARY KEY, 
  FirstName varchar(32)
);
GO

-- purposely out of alphabetical order
INSERT dbo.what(ID,FirstName) 
  VALUES(1,'mike'),(2,'grant'),(3,'aaron');

-- ordered by primary key, since cheapest 
-- method is CI scan
SELECT ID,FirstName FROM dbo.what;

-- now, someone else creates an index
CREATE INDEX x ON dbo.what(FirstName);

-- cheapest method changed; now index scan is used, 
-- ordered alpha:
SELECT ID,FirstName FROM dbo.what;

-- TOP 100 PERCENT doesn't help:
SELECT ID,FirstName FROM (SELECT TOP (100) PERCENT 
  ID,FirstName FROM dbo.what ORDER BY ID) AS sub;

-- neither does adding implicit sorting in a CTE:
;WITH cte AS (SELECT ID,FirstName, 
 rn = ROW_NUMBER() OVER (ORDER BY ID) FROM dbo.what)
SELECT ID,FirstName FROM cte;

-- unless the window function is materialized (but this still isn't guaranteed):
;WITH cte AS (SELECT ID,FirstName,
  rn = ROW_NUMBER() OVER (ORDER BY ID) FROM dbo.what)
SELECT ID,FirstName,rn FROM cte;

-- also, don't use this lazy shorthand
-------------------------------------------v
SELECT ID,FirstName FROM dbo.what ORDER BY 1;

DROP TABLE dbo.what;
GO







/* evaluation order */

CREATE TABLE dbo.MaybeNumbers
(
  ID int, Score varchar(12)
);

CREATE TABLE dbo.Related
(
  ID int
);

INSERT dbo.MaybeNumbers(ID, Score) 
  VALUES(1,'100'),--(2,'Aaron'),
  (5,'45');

INSERT dbo.Related(ID) VALUES(1), (5), (16), (489);
GO

SELECT m.ID, m.Score * 5
  FROM dbo.MaybeNumbers AS m
  INNER JOIN dbo.Related AS r
  ON m.ID = r.ID;
GO

;WITH OnlyNumbers AS
(
  SELECT ID, Score
  FROM dbo.MaybeNumbers
  WHERE ISNUMERIC(Score) = 1
  --WHERE Value NOT LIKE '%[^0-9]%' 
)
SELECT ID, Score
  FROM OnlyNumbers
  WHERE Score > 10;
GO

-- how to solve?

SELECT m.ID, 5 * CASE WHEN ISNUMERIC(m.Score) = 1 
      THEN m.Score 
      ELSE NULL 
      END,
    TRY_CONVERT(int, m.Score) * 5
  FROM dbo.MaybeNumbers AS m
  INNER JOIN dbo.Related AS r
  ON m.ID = r.ID;
GO

DROP TABLE dbo.MaybeNumbers, dbo.Related;